﻿// ----------------------------------------------------------------------------------------
//
// TxExplorer de Luxe                                                Ulrich Müller 05.02.10
//                                                                   Aspire C# 2008 Express
// Betrieb des Basismodells (TX Explorer) mit dem Joystick an winmm.dll
// Rechtes Rad an M1, Linkes Rad an M2
// Scheinwerfer an M4, Blaulicht an M3, Photowiderstand an I1
// 
// ----------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using FishFaceTX;

namespace TxExplorerLuxus {
  public partial class MainExLuxus : Form {
    struct FahrAuftrag {
      public Dir rDir;
      public int rSpeed;
      public Dir lDir;
      public int lSpeed;
      public FahrAuftrag(Dir rD, int rS, Dir lD, int lS) {
        rDir   = rD;
        rSpeed = rS;
        lDir   = lD;
        lSpeed = lS;
      }
    }

    FishFace tx       = new FishFace();
    FahrAuftrag[,] fa = new FahrAuftrag[3, 3];
    bool BlauAn       = false;

    const Mot mRechteRaupe = Mot.M1;
    const Mot mLinkeRaupe  = Mot.M2;
    const Dir dVor         = Dir.Left;
    const Dir dRuck        = Dir.Right;
    const int SpeedOff     = 0;
    const int SpeedFull    = 8;
    const int SpeedHalf    = 6;
    const Unv uPhotoW      = Unv.I1;
    const Out oBlauL       = Out.O5;
    const int LampenAn     = 555;
    const int LampenAus    = 500;
    const Out oLampen      = Out.O7;

    # region --- Joystick Definitionen -----------
    [DllImport("winmm.dll")]
    private static extern uint joyGetPosEx(uint uJoyID, ref JoyInfoEx ji);

    private const uint JOYERR_NOERROR = 0;
    private const uint JoyReturnButtons = 0x80;
    private const uint JoyReturnCentered = 0x400;
    private const uint JoyReturnPOV = 0x40;
    private const uint JoyReturnR = 0x8;
    private const uint JoyReturnU = 0x10;
    private const uint JoyReturnV = 0x20;
    private const uint JoyReturnX = 0x1;
    private const uint JoyReturnY = 0x2;
    private const uint JoyReturnZ = 0x4;
    private const uint JoyReturnAll = JoyReturnX | JoyReturnY | JoyReturnZ
                                         | JoyReturnR | JoyReturnU | JoyReturnV
                                         | JoyReturnPOV | JoyReturnButtons;
    private struct JoyInfoEx {
      public uint dwSize;
      public uint dwFlags;
      public uint dwXpos;
      public uint dwYpos;
      public uint dwZpos;
      public uint dwRpos;
      public uint dwUpos;
      public uint dwVpos;
      public uint dwButtons;
      public uint dwButtonNumber;
      public uint dwPOV;
      public uint dwReserved1;
      public uint dwReserved2;
    }
    # endregion

    private JoyInfoEx ji = new JoyInfoEx();

    public MainExLuxus() {
      InitializeComponent();
      // X links, Y vor, mitte, hinten
      fa[0, 0] = new FahrAuftrag(dVor,  SpeedFull, dVor,  SpeedHalf);
      fa[0, 1] = new FahrAuftrag(dVor,  SpeedHalf, dRuck, SpeedHalf);
      fa[0, 2] = new FahrAuftrag(dRuck, SpeedHalf, dRuck, SpeedFull);
      // X mitte, Y vor, mitte, hinten
      fa[1, 0] = new FahrAuftrag(dVor,    SpeedFull, dVor,    SpeedFull);
      fa[1, 1] = new FahrAuftrag(Dir.Off, SpeedOff,  Dir.Off, SpeedOff);
      fa[1, 2] = new FahrAuftrag(dRuck,   SpeedFull, dRuck,   SpeedFull);
      // X rechts, Y vor, mitte, hinten
      fa[2, 0] = new FahrAuftrag(dVor,  SpeedHalf, dVor,  SpeedFull);
      fa[2, 1] = new FahrAuftrag(dRuck, SpeedHalf, dVor,  SpeedHalf);
      fa[2, 2] = new FahrAuftrag(dRuck, SpeedHalf, dRuck, SpeedFull);

      try {
        tx.OpenController("COM25");             // --- zutreffender COM-Name -> RoboTxTest
        lblStatus.Text = "Explorer ready for Action";
      }
      catch (FishFaceException eft) {
        lblStatus.Text = eft.Message;
      }
    }

    private void MainExLuxus_FormClosed(object sender, FormClosedEventArgs e) {
      tmrBlau.Enabled = false;
      tx.CloseController();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      ji.dwSize = 52;
      ji.dwFlags = JoyReturnAll;
      lblHinweis.Text = "Beenden durch ESC-Taste";
      int xFa, yFa, zFa, hell;

      while (!tx.Finish()) {
        
        // Joystick X, Y und Z liefern Werte zwischen 0 - 65535, Mitte 32767
        if (joyGetPosEx(0, ref ji) != JOYERR_NOERROR) {
          lblStatus.Text = "Kein Joystick da";
          return;
        }
        
        // Joystick X normieren auf links 0, mitte 1 und rechts 2
        xFa = (int)(ji.dwXpos / 29000);
        
        // Joystick Y normieren auf vor 0, mitte 1 und hinten 2
        yFa = (int)(ji.dwYpos / 29000);
        
        // Basisgeschwindigkeit 256 + variabler Teil aus Joystick Z mit vertauschten Werten
        zFa = 32 - ((int)(ji.dwZpos / 2048));
        
        // --- Setzen der Fahraufträge ---
        tx.SetMotor(mRechteRaupe, fa[xFa, yFa].rDir, fa[xFa, yFa].rSpeed * zFa + 256);
        tx.SetMotor(mLinkeRaupe, fa[xFa, yFa].lDir, fa[xFa, yFa].lSpeed * zFa + 256);
        
        // --- Schalten Blaulicht -----------
        if ((ji.dwButtons & (uint)0x2) > 0) {
          do {
            tx.Pause(55);
            joyGetPosEx(0, ref ji);
          } while ((ji.dwButtons & (uint)0x2) > 0);
          if (tmrBlau.Enabled) {
            tmrBlau.Enabled = false;
            tx.SetLamp(oBlauL, Dir.Off);
          } else tmrBlau.Enabled = true;
        }

        // --- Schalten des Fahrlichtes in Abhängigkeit der Umgebungshelligkeit ----
        hell = tx.GetAnalog(uPhotoW);
        lblHell.Text = "Helligkeitswert : " + hell.ToString();
        if (hell > LampenAn) tx.SetLamp(oLampen, Dir.On);
        else if (hell < LampenAus) tx.SetLamp(oLampen, Dir.Off);
        
        // --- Allgemeine Abfragepause ---
        tx.Pause(123);
      }
      lblHinweis.Text = "Das war's";
    }

    // --- Blaulicht-Blinker ----------------------
    private void tmrBlau_Tick(object sender, EventArgs e) {
      BlauAn = !BlauAn;
      if (BlauAn) tx.SetLamp(oBlauL, Dir.On);
      else tx.SetLamp(oBlauL, Dir.Off);
    }
  }
}
